import numpy as np
import matplotlib.pyplot as plt
import scipy.io as sio
from scipy.signal import stft, cwt, ricker, morlet
from signal_processing.hht import hsa
from time import time

def process_training_data(subject_name,sensors,method):
    print('Preprocessing data for subject \'{}\'...'.format(subject_name))
    fs = 100 # Hz
    widths = np.arange(0.1,50,0.5)

    print('{} Storing data as ndarray...'.format(subject_name))
    start = time()

    num_samples = 0
    with open('data/ascii/data_set_IVa_{}_cnt.txt'.format(subject_name)) as f:
        for _ in f:
            num_samples += 1
    t = np.linspace(0,num_samples/fs,num_samples)

    # Store raw data as ndarray
    raw_data = np.zeros((118,num_samples))
    data_file = open('data/ascii/data_set_IVa_{}_cnt.txt'.format(subject_name))
    for i,line in enumerate(data_file):
        raw_data[:,i] = np.array([float(x) for x in line.split()])
    print('{} Stored data in {:.2f} seconds'.format(subject_name,time() - start))
    raw_array = np.zeros((280,350,len(sensors)))
    processing_time_array = np.zeros(280)

    if method == 'hht':
        feature_array = np.zeros((280,100,350,len(sensors)))
    elif method == 'cwt':
        feature_array = np.zeros((280,100,349,len(sensors)))
    elif method == 'stft':
        feature_array = np.empty((280,17,23,len(sensors)))
    label_array = np.zeros((280), dtype=int)

    label_file = open('data/ascii/data_set_IVa_{}_mrk.txt'.format(subject_name))
    true_labels = sio.loadmat('data/mat/true_labels_{}.mat'.format(subject_name))
    true_y = true_labels['true_y'].flatten()
    CMA = 0.0 # Cumulative Moving Average
    for i,line in enumerate(label_file):
        print('{} Processing sequence {}/280 ({:.4f} s/step)'.format(subject_name,i+1,CMA))

        label_array[i] = true_y[i] - 1

        t0_idx = int(line.split()[0]) # Start of cue
        t1_idx = int(t0_idx + 3.5*fs) # Each cue remains for 3.5s

        data = raw_data[:,t0_idx:t1_idx]
        start_seq = time()
        for j,sensor in enumerate(sensors):
            raw_array[i,:,j] = data[sensor]
            feature_matr = None
            if method == 'hht':
                feature_matr = hsa.hht(data[sensor],fs,widths)
            elif method == 'stft':
                _, _, Zxx = stft(data[sensor],fs,nperseg=32)
                feature_matr = np.abs(Zxx)
            elif method == 'cwt':
                feature_matr = cwt(data[sensor,:-1],morlet,widths)
            feature_array[i,:,:,j] = feature_matr
        dt = time() - start_seq
        CMA = (dt + i*CMA)/(i+1)
        processing_time_array[i] = dt
    np.save('data/raw/{}.npy'.format(subject_name),raw_array)
    return feature_array, label_array, np.mean(processing_time_array)

def main():
    start = time()

    method = 'hht'

    # Use sensors at Fp1, Fp2, F3, F4, C3, C4, O1 and O2
    sensors = [0,4,15,19,51,55,111,112]

    if method == 'hht':
        features = np.empty((0,100,350,len(sensors)))
    elif method == 'cwt':
        features = np.empty((0,100,349,len(sensors)))
    elif method == 'stft':
        features = np.empty((0,17,23,len(sensors)))
    labels = np.empty((0,))

    subjects = ['aa','al','av','aw','ay']
    total_processing_time = 0.0
    for name in subjects:
        new_features, new_labels, processing_time = process_training_data(name,sensors,method)
        total_processing_time += processing_time
        features = np.vstack((features, new_features))
        labels = np.hstack((labels, new_labels))
    print('Average processing time = {}'.format(total_processing_time / 5))

    np.save('data/processed/{}/features.npy'.format(method), features)
    np.save('data/processed/labels.npy', labels)

    print('Finished!')
    print('Total time: {:.2f} seconds'.format(time() - start))

if __name__ == '__main__':
    main()
