import numpy as np
from scipy.interpolate import CubicSpline
from scipy.signal import argrelmax, argrelmin

from signal_processing.hht.utils import count_extrema, count_zero_crossings


def decompose(x, t=None, max_modes=10):
    imfs = np.array([])

    residue = x.copy()
    for _ in range(max_modes):
        imf = sift(residue, t)
        if imfs.any():
            imfs = np.vstack((imfs, imf))
        else:
            imfs = imf

        residue -= imf

        num_extrema = count_extrema(residue)
        if num_extrema < 3:
            break

    imfs = np.vstack((imfs, residue))
    return imfs

def sift(residue, t=None, max_siftings=200):
    mode = residue.copy()

    prev_extrema = 0
    prev_zero_crossings = 0
    s_counter = 0

    for n in range(max_siftings):
        upper_envelope, lower_envelope = get_envelopes(mode, t)
        mode -= (upper_envelope + lower_envelope) / 2.0

        num_extrema = count_extrema(mode)
        num_zero_crossings = count_zero_crossings(mode)

        if abs(num_extrema - num_zero_crossings) <= 1 and -0.001 <= np.mean(mode) <= 0.001:
            if prev_extrema == num_extrema and prev_zero_crossings == num_zero_crossings:
                s_counter += 1
            else:
                s_counter = 0
            
            prev_extrema = num_extrema
            prev_zero_crossings = num_zero_crossings

            if s_counter >= 5:
                break

    num_extrema = count_extrema(mode)
    if num_extrema > 1:
        for _ in range(5):
            upper_envelope, lower_envelope = get_envelopes(mode)
            mode -= (upper_envelope + lower_envelope) / 2.0

    return mode

def get_envelopes(y, x=None):
    if x is None:
        x = np.arange(len(y))

    upper_envelope = None
    lower_envelope = None

    for upper in (True,False):
        if upper:
            extrema = argrelmax(y)[0]
            # At least two extrema are needed for linear interpolation
            if np.size(extrema) == 0:
                upper_envelope = np.zeros(len(y))
                continue
            if np.size(extrema) == 1:
                upper_envelope = np.ones(len(y)) * y[extrema]
                continue
        else:
            extrema = argrelmin(y)[0]
            # At least two extrema are needed for linear interpolation
            if np.size(extrema) == 0:
                lower_envelope = np.zeros(len(y))
                continue
            if np.size(extrema) == 1:
                lower_envelope = np.ones(len(y)) * y[extrema]
                continue

        interpolation_points = y[extrema]

        # Use linear interpolation for start point

        x1, x2 = x[extrema[0]], x[extrema[1]]
        y1, y2 = y[extrema[0]], y[extrema[1]]

        slope = (y1 - y2)/(x1 - x2) 

        if upper:
            start_point = max(slope*(x[0] - x1) + y1, y[0])
        else:
            start_point = min(slope*(x[0] - x1) + y1, y[0])

        interpolation_points = np.insert(interpolation_points, [0], [start_point])
        extrema = np.insert(extrema, [0], [0])

        # And for end point

        x1, x2 = x[extrema[-1]], x[extrema[-2]]
        y1, y2 = y[extrema[-1]], y[extrema[-2]]

        slope = (y1 - y2)/(x1 - x2)

        if upper:
            end_point = max(slope*(x[-1] - x1) + y1, y[-1])
        else:
            end_point = min(slope*(x[-1] - x1) + y1, y[-1])

        interpolation_points = np.append(interpolation_points, [end_point])
        extrema = np.append(extrema, [len(y)-1])

        if upper:
            upper_envelope = CubicSpline(x[extrema],interpolation_points)(x)
        else:
            lower_envelope = CubicSpline(x[extrema],interpolation_points)(x)

    return upper_envelope, lower_envelope
