import numpy as np
import matplotlib.pyplot as plt
from scipy.signal import hilbert, medfilt
from sys import maxsize

def plot_emd(x,imfs,t=None,title=None):
    if t is None:
        t = np.arange(len(x))
    num_imfs = len(imfs)
    fig, ax = plt.subplots(num_imfs+1,1,sharex=True,sharey=True)
    if title is not None:
        fig.suptitle(title)
    ax[0].plot(t,x)
    ax[0].set_ylabel('Signal')
    for i,imf in enumerate(imfs):
        ax[i+1].plot(t,imf)
        ax[i+1].set_ylabel('IMF {}'.format(i+1))
    ax[-1].set_xlabel('Time [s]')
    ax[-1].set_ylabel('Residue')
    plt.show()

def plot_emd_ps(x,imfs,fs,title=None,max_freq=maxsize):
    num_imfs = len(imfs)
    fig, ax = plt.subplots(num_imfs-1,1,sharex=True)
    if title is not None:
        fig.suptitle(title)
    for i,imf in enumerate(imfs[:-1]):
        freq, ps = to_power_spectrum(imf,1.0/fs)
        ax[i].plot(freq,ps)
        ax[i].set_xlim(0,min(fs/2, max_freq)+1)
        ax[i].set_ylabel('IMF {}'.format(i+1))
    ax[-1].set_xlabel('Frequency [Hz]')
    plt.show()

def plot_emd_inst_ampl(x,imfs,t=None,title=None):
    if t is None:
        t = np.arange(len(x))
    num_imfs = len(imfs)
    fig, ax = plt.subplots(num_imfs+1,1,sharex=True)
    if title is not None:
        fig.suptitle(title)
    ax[0].plot(t,x)
    ax[0].set_ylabel('Signal')
    for i,imf in enumerate(imfs):
        ax[i+1].plot(t,medfilt(np.abs(hilbert(imf)),kernel_size=125))
    plt.show()

def plot_emd_inst_freq(x,imfs,fs,t=None,title=None):
    if t is None:
        t = np.arange(len(x))
    num_imfs = len(imfs)
    fig, ax = plt.subplots(num_imfs-1,1,sharex=True,sharey=True)
    if title is not None:
        fig.suptitle(title)
    for i,imf in enumerate(imfs[:-1]):
        analytic = hilbert(imf)
        inst_phase = np.unwrap(np.angle(analytic)) * fs/(2*np.pi)
        inst_freq = np.diff(inst_phase)
        ax[i].plot(t[1:],inst_freq)
        ax[i].set_ylabel('IMF {}'.format(i+1))
    ax[-1].set_xlabel('Time[s]')
    plt.show()

def plot_two_emds(x1,imfs1,x2,imfs2,t=None,suptitle=None,left_title=None,right_title=None):
    if t is None:
        t = np.arange(len(x1))
    num_imfs = max(len(imfs1),len(imfs2))
    fig, ax = plt.subplots(num_imfs+1,2,sharex='col',sharey='row')
    if suptitle is not None:
        fig.suptitle(suptitle)
    ax[0,0].plot(t,x1)
    #ax[0,0].grid(True)
    if left_title is not None:
        ax[0,0].set_title(left_title)
    ax[0,0].set_ylabel('Signal')
    ax[0,1].plot(t,x2)
    #ax[0,1].grid(True)
    if right_title is not None:
        ax[0,1].set_title(right_title)
    ax[0,1].set_ylabel('Signal')
    for i in range(num_imfs):
        if len(imfs1) > i:
            ax[i+1,0].plot(t,imfs1[i])
            #ax[i+1,0].grid(True)
        if len(imfs2) > i:
            ax[i+1,1].plot(t,imfs2[i])
            #ax[i+1,1].grid(True)
    plt.show()

def plot_two_emds_ps(x1,imfs1,x2,imfs2,fs,suptitle=None,left_title=None,right_title=None,max_freq=maxsize):
    num_imfs = max(len(imfs1),len(imfs2))
    fig, ax = plt.subplots(num_imfs+1,2,sharey="row",sharex="col")
    if suptitle is not None:
        fig.suptitle(suptitle)
    freq, ps = to_power_spectrum(x1,1.0/fs)
    ax[0,0].plot(freq,ps)
    ax[0,0].set_xlim(0,min(fs/2,max_freq))
    if left_title is not None:
        ax[0,0].set_title(left_title)
    ax[0,0].set_ylabel('Signal')
    freq, ps = to_power_spectrum(x2,1.0/fs)
    ax[0,1].plot(freq,ps)
    ax[0,1].set_xlim(0,min(fs/2, max_freq))
    if right_title is not None:
        ax[0,1].set_title(right_title)
    ax[0,1].set_ylabel('Signal')
    for i in range(num_imfs):
        if len(imfs1) > i:
            freq, ps = to_power_spectrum(imfs1[i],1.0/fs)
            ax[i+1,0].plot(freq,ps)
            ax[i+1,0].set_xlim(0,min(fs/2, max_freq))
        if len(imfs2) > i:
            freq, ps = to_power_spectrum(imfs2[i],1.0/fs)
            ax[i+1,1].plot(freq,ps)
            ax[i+1,1].set_xlim(0,min(fs/2, max_freq))
    plt.show()

def plot_two_emds_inst_freq(x1,imfs1,x2,imfs2,fs,t=None,suptitle=None,left_title=None,right_title=None):
    if t is None:
        t = np.arange(len(imfs[1]))
    num_imfs = max(len(imfs1),len(imfs2))
    fig, ax = plt.subplots(num_imfs+1,2,sharey="row",sharex="col")
    if suptitle is not None:
        fig.suptitle(suptitle)
    hs = hilbert(x1)
    omega = np.unwrap(np.angle(hs))
    ax[0,0].plot(t[1:],fs/(2.0*np.pi)*np.diff(omega))
    #ax[0,0].grid(True)
    if left_title is not None:
        ax[0,0].set_title(left_title)
    ax[0,0].set_ylabel('Signal')
    hs = hilbert(x2)
    omega = np.unwrap(np.angle(hs))
    ax[0,1].plot(t[1:],fs/(2.0*np.pi)*np.diff(omega))
    #ax[0,1].grid(True)
    if right_title is not None:
        ax[0,1].set_title(right_title)
    ax[0,1].set_ylabel('Signal')
    for i in range(num_imfs):
        if len(imfs1) > i:
            hs = hilbert(imfs1[i])
            omega = np.unwrap(np.angle(hs))
            ax[i+1,0].plot(t[1:],fs/(2.0*np.pi)*np.diff(omega))
            #ax[i+1,0].grid(True)
        if len(imfs2) > i:
            hs = hilbert(imfs2[i])
            omega = np.unwrap(np.angle(hs))
            ax[i+1,1].plot(t[1:],fs/(2.0*np.pi)*np.diff(omega))
            #ax[i+1,1].grid(True)
    plt.show()

def plot_two_emds_inst_ampl(x1,imfs1,x2,imfs2,t=None,suptitle=None,left_title=None,right_title=None):
    if t is None:
        t = np.arange(len(imfs1[0]))
    num_imfs = max(len(imfs1),len(imfs2))
    fig, ax = plt.subplots(num_imfs+1,2,sharey="row",sharex="col")
    if suptitle is not None:
        fig.suptitle(suptitle)
    ax[0,0].plot(t,x1)
    #ax[0,0].grid(True)
    if left_title is not None:
        ax[0,0].set_title(left_title)
    ax[0,0].set_ylabel('Signal')
    ax[0,1].plot(t,x2)
    #ax[0,1].grid(True)
    if right_title is not None:
        ax[0,1].set_title(right_title)
    ax[0,1].set_ylabel('Signal')
    for i in range(num_imfs):
        if len(imfs1) > i:
            ax[i+1,0].plot(t,medfilt(np.abs(hilbert(imfs1[i])),kernel_size=125))
            #ax[i+1,0].grid(True)
        if len(imfs2) > i:
            ax[i+1,1].plot(t,medfilt(np.abs(hilbert(imfs2[i])),kernel_size=125))
            #ax[i+1,1].grid(True)
    plt.show()

def to_power_spectrum(x,dt):
    ps = np.abs(np.fft.fft(x))**2
    freqs = np.fft.fftfreq(len(x),dt)
    idx = np.argsort(freqs)
    return freqs[idx],ps[idx]
